@@ -32,14 +32,13 @@ from gajim.common.i18n import _
 from gajim.common.util.text import process_non_spacing_marks
 
 from gajim.gtk.assistant import Assistant
+from gajim.gtk.assistant import AssistantErrorPage
+from gajim.gtk.assistant import AssistantPage
+from gajim.gtk.assistant import AssistantProgressPage
 from gajim.gtk.assistant import DefaultPage
-from gajim.gtk.assistant import ErrorPage
-from gajim.gtk.assistant import Page
-from gajim.gtk.assistant import ProgressPage
 from gajim.gtk.dataform import DataFormWidget
 from gajim.gtk.util.misc import container_remove_all
 from gajim.gtk.util.misc import ensure_not_destroyed
-from gajim.gtk.widgets import MultiLineLabel
 
 log = logging.getLogger("gajim.gtk.adhoc")
 
@@ -83,7 +82,6 @@ class AdHocCommands(Assistant):
         self._client.get_module("AdHocCommands").request_command_list(
             self.jid, callback=self._received_command_list
         )
-        self.show_all()
 
     @overload
     def get_page(self, name: Literal["request"]) -> RequestCommandList: ...
@@ -106,7 +104,7 @@ class AdHocCommands(Assistant):
     @overload
     def get_page(self, name: Literal["executing"]) -> Executing: ...
 
-    def get_page(self, name: str) -> Page:
+    def get_page(self, name: str) -> AssistantPage:
         return self._pages[name]
 
     @ensure_not_destroyed
@@ -164,6 +162,7 @@ class AdHocCommands(Assistant):
         self.show_page("end")
 
     def _cleanup(self) -> None:
+        Assistant._cleanup(self)
         self._destroyed = True
 
     def _on_button_clicked(self, _assistant: AdHocCommands, button_name: str) -> None:
@@ -219,13 +218,13 @@ class AdHocCommands(Assistant):
         self.show_page("commands")
 
 
-class Commands(Page):
+class Commands(AssistantPage):
     __gsignals__ = {
         "execute": (GObject.SignalFlags.RUN_LAST, None, ()),
     }
 
     def __init__(self) -> None:
-        Page.__init__(self)
+        AssistantPage.__init__(self)
 
         self.set_valign(Gtk.Align.FILL)
         self.complete = True
@@ -286,9 +285,9 @@ class Commands(Page):
         return ["execute"]
 
 
-class Stage(Page):
+class Stage(AssistantPage):
     def __init__(self) -> None:
-        Page.__init__(self)
+        AssistantPage.__init__(self)
 
         self.set_valign(Gtk.Align.FILL)
         self.complete = False
@@ -371,9 +370,9 @@ class Stage(Page):
         return self._last_stage_data.default.value
 
 
-class Completed(Page):
+class Completed(AssistantPage):
     def __init__(self) -> None:
-        Page.__init__(self)
+        AssistantPage.__init__(self)
 
         self.set_valign(Gtk.Align.FILL)
         self.complete = True
@@ -395,16 +394,8 @@ class Completed(Page):
         self._icon_text.append(self._label)
         self.append(self._icon_text)
 
-        self._notes = Gtk.Grid(row_spacing=6, column_spacing=12)
-        self._notes.insert_column(0)
-        self._notes.insert_column(1)
-        self._notes.set_vexpand(False)
-        self._notes.set_hexpand(True)
-        notes_box = Gtk.Box()
-        notes_box.set_halign(Gtk.Align.CENTER)
-        notes_box.set_vexpand(False)
-        notes_box.append(self._notes)
-        self.append(notes_box)
+        self._notes = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
+        self.append(self._notes)
 
     def process_stage(self, stage_data: AdHocCommand) -> None:
         self._reset_severity()
@@ -449,18 +440,23 @@ class Completed(Page):
         if notes is None:
             return
 
-        for i, note in enumerate(notes):
+        for note in notes:
+            box = Gtk.Box(margin_start=24, spacing=24)
             if len(notes) > 1:
-                icon = SeverityIcon(note.type)
-                icon.set_visible(True)
-                self._notes.attach(icon, 0, i, 1, 1)
+                box.append(SeverityIcon(note.type))
+            else:
+                box.set_halign(Gtk.Align.CENTER)
 
-            label = MultiLineLabel(label=process_non_spacing_marks(note.text))
-            label.set_justify(Gtk.Justification.CENTER)
-            label.set_vexpand(False)
-            label.set_visible(True)
-            self._notes.attach(label, 1, i, 1, 1)
+            label = Gtk.Label(
+                label=process_non_spacing_marks(note.text),
+                natural_wrap_mode=Gtk.NaturalWrapMode.WORD,
+                selectable=True,
+                wrap=True,
+                wrap_mode=Pango.WrapMode.WORD_CHAR,
+            )
+            box.append(label)
 
+            self._notes.append(box)
             self._bump_severity(note.type)
 
     def _show_icon(self, show: bool):
@@ -482,9 +478,9 @@ class Completed(Page):
         return ["commands"]
 
 
-class Error(ErrorPage):
+class Error(AssistantErrorPage):
     def __init__(self) -> None:
-        ErrorPage.__init__(self)
+        AssistantErrorPage.__init__(self)
 
         self._show_commands_button = False
         self.set_heading(_("An error occurred"))
@@ -504,16 +500,16 @@ class End(DefaultPage):
         self.title = _("Completed")
 
 
-class Executing(ProgressPage):
+class Executing(AssistantProgressPage):
     def __init__(self) -> None:
-        ProgressPage.__init__(self)
+        AssistantProgressPage.__init__(self)
         self.set_title(_("Executing…"))
         self.set_text(_("Executing…"))
 
 
-class RequestCommandList(ProgressPage):
+class RequestCommandList(AssistantProgressPage):
     def __init__(self) -> None:
-        ProgressPage.__init__(self)
+        AssistantProgressPage.__init__(self)
         self.set_title(_("Requesting Command List"))
         self.set_text(_("Requesting Command List"))
 
