@@ -55,8 +55,7 @@ from gajim.common.const import GAJIM_FAQ_URI
 from gajim.common.const import GAJIM_PRIVACY_POLICY_URI
 from gajim.common.const import GAJIM_SUPPORT_JID
 from gajim.common.const import GAJIM_WIKI_URI
-from gajim.common.helpers import dump_json
-from gajim.common.helpers import load_json
+from gajim.common.helpers import idle_add_once
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import ResourceContact
@@ -71,7 +70,7 @@ from gajim.gtk.const import APP_ACTIONS
 from gajim.gtk.const import FEATURE_ACCOUNT_ACTIONS
 from gajim.gtk.const import MuteState
 from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
-from gajim.gtk.const import SHORTCUTS
+from gajim.gtk.shortcut_manager import ShortcutManager
 from gajim.gtk.util.icons import get_icon_theme
 from gajim.gtk.util.misc import open_file
 from gajim.gtk.util.misc import open_uri
@@ -203,7 +202,9 @@ class GajimApplication(Adw.Application, CoreApplication):
         self.connect("activate", self._on_activate)
         self.connect("handle-local-options", self._handle_local_options)
         self.connect("command-line", self._command_line)
-        self.connect("shutdown", self._shutdown)
+        self.connect("shutdown", self._on_shutdown)
+        self.connect("query-end", self._on_query_end)
+        self.connect("window_added", self._on_window_added)
 
     @staticmethod
     def _get_remaining_entry():
@@ -219,7 +220,7 @@ class GajimApplication(Adw.Application, CoreApplication):
 
     def _startup(self) -> None:
         if sys.platform in ("win32", "darwin"):
-            # Changing the PANGOCAIRO_BACKEND is necessary on Windows/MacOS
+            # Changing the PANGOCAIRO_BACKEND is necessary on Windows/macOS
             # to render colored emoji glyphs
             os.environ["PANGOCAIRO_BACKEND"] = "fontconfig"
 
@@ -232,6 +233,7 @@ class GajimApplication(Adw.Application, CoreApplication):
         icon_theme.add_search_path(str(configpaths.get("ICONS")))
 
         self.avatar_storage = AvatarStorage()
+        self._shortcut_manager = ShortcutManager()
 
         app.load_css_config()
 
@@ -256,7 +258,6 @@ class GajimApplication(Adw.Application, CoreApplication):
         for account in accounts:
             self.add_account_actions(account)
 
-        self._load_shortcuts()
         self.update_app_actions_state()
 
         self.register_event("feature-discovered", ged.CORE, self._on_feature_discovered)
@@ -278,14 +279,34 @@ class GajimApplication(Adw.Application, CoreApplication):
                 body_use_markup=True,
             )
 
+        self._inhibit_cookie = self.inhibit(
+            app.window, Gtk.ApplicationInhibitFlags.LOGOUT, _("Quitting…")
+        )
+
         GLib.timeout_add(100, self._auto_connect)
 
-    def _shutdown(self, _application: GajimApplication) -> None:
-        self._shutdown_core()
+    def _on_window_added(
+        self, _application: GajimApplication, window: Gtk.Window
+    ) -> None:
+        self._shortcut_manager.install_shortcuts(window, ["app"])
 
-    def _quit_app(self) -> None:
+    def start_shutdown(self) -> None:
+        app.window.start_shutdown()
+        CoreApplication._start_shutdown(self)
+
+    def _shutdown_complete(self) -> None:
+        CoreApplication._shutdown_complete(self)
+        self.uninhibit(self._inhibit_cookie)
         self.quit()
 
+    def _on_query_end(self, _application: GajimApplication) -> None:
+        app.log("app").info("Session end signal received")
+        # Use a idle call so Gtk can send QueryEndResponse on Linux
+        idle_add_once(self.start_shutdown)
+
+    def _on_shutdown(self, _application: GajimApplication) -> None:
+        app.log("app").info("Shutdown Application")
+
     def _command_line(
         self, _application: GajimApplication, command_line: Gio.ApplicationCommandLine
     ) -> int:
@@ -386,6 +407,9 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _on_activate(self, _application: Gtk.Application) -> None:
         app.window.show_window()
 
+    def get_shortcut_manager(self) -> ShortcutManager:
+        return self._shortcut_manager
+
     def _add_app_actions(self) -> None:
         for action in APP_ACTIONS:
             action_name, variant = action
@@ -503,40 +527,6 @@ class GajimApplication(Adw.Application, CoreApplication):
         enabled_accounts = bool(app.settings.get_active_accounts())
         self.set_action_state("start-chat", enabled_accounts)
 
-    @staticmethod
-    def get_user_shortcuts() -> dict[str, list[str]]:
-        user_path = configpaths.get("MY_SHORTCUTS")
-        user_shortcuts: dict[str, list[str]] = {}
-        if user_path.exists():
-            app.log("app").info("Load user shortcuts")
-            user_shortcuts = load_json(user_path, default={})
-
-        return user_shortcuts
-
-    def set_user_shortcuts(self, user_shortcuts: dict[str, list[str]]) -> None:
-        user_path = configpaths.get("MY_SHORTCUTS")
-        dump_json(user_path, user_shortcuts)
-        self._load_shortcuts()
-
-    def _load_shortcuts(self) -> None:
-        shortcuts = {
-            action_name: data.accelerators for action_name, data in SHORTCUTS.items()
-        }
-        no_rebind_allowed = [
-            action_name
-            for action_name, data in SHORTCUTS.items()
-            if not data.allow_rebind
-        ]
-
-        user_shortcuts = self.get_user_shortcuts()
-        for action_name in no_rebind_allowed:
-            shortcuts.pop(action_name, None)
-
-        shortcuts.update(user_shortcuts)
-
-        for action, accelerators in shortcuts.items():
-            self.set_accels_for_action(action, accelerators)
-
     def _on_feature_discovered(self, event: events.FeatureDiscovered) -> None:
         self.update_feature_actions_state(event.account)
 
@@ -617,7 +607,7 @@ class GajimApplication(Adw.Application, CoreApplication):
 
     @staticmethod
     def _on_quit_action(_action: Gio.SimpleAction, _param: GLib.Variant | None) -> None:
-        app.window.quit()
+        app.app.start_shutdown()
 
     @staticmethod
     def _on_new_chat_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
