@@ -7,13 +7,13 @@ from __future__ import annotations
 import logging
 import sys
 import uuid
-from urllib.parse import urlparse
 
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.task import Task
 
@@ -25,6 +25,7 @@ from gajim.common.commands import ChatCommands
 from gajim.common.const import CallType
 from gajim.common.const import Display
 from gajim.common.ged import EventHelper
+from gajim.common.helpers import idle_add_once
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
@@ -35,10 +36,7 @@ from gajim.common.storage.archive.models import Message
 from gajim.common.structs import OutgoingMessage
 from gajim.common.types import ChatContactT
 from gajim.common.util.muc import message_needs_highlight
-from gajim.common.util.preview import filename_from_uri
-from gajim.common.util.preview import format_geo_coords
-from gajim.common.util.preview import guess_simple_file_type
-from gajim.common.util.preview import split_geo_uri
+from gajim.common.util.preview import get_preview_data
 from gajim.common.util.text import remove_invalid_xml_chars
 
 from gajim.gtk.activity_page import ActivityPage
@@ -46,11 +44,15 @@ from gajim.gtk.alert import InformationAlertDialog
 from gajim.gtk.chat_banner import ChatBanner
 from gajim.gtk.chat_function_page import ChatFunctionPage
 from gajim.gtk.chat_function_page import FunctionMode
+from gajim.gtk.chat_list_row import ChatListRow
 from gajim.gtk.control import ChatControl
 from gajim.gtk.message_actions_box import MessageActionsBox
 from gajim.gtk.message_input import MessageInputTextView
+from gajim.gtk.preview.preview import PreviewWidget
+from gajim.gtk.sidebar_listbox import SideBarListBoxRow
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import allow_send_message
+from gajim.gtk.util.misc import gi_gui_package_version
 from gajim.gtk.util.window import open_window
 
 log = logging.getLogger("gajim.gtk.chatstack")
@@ -87,7 +89,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         box.append(self._chat_banner)
-        box.append(Gtk.Separator(margin_start=6, margin_end=6))
+        box.append(Gtk.Separator(margin_start=12, margin_end=12))
         box.append(self._chat_control.get_widget())
         box.append(self._message_action_box)
 
@@ -142,6 +144,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
                 ("account-connected", 85, self._on_account_state),
                 ("account-disconnected", 85, self._on_account_state),
                 ("register-actions", ged.GUI1, self._on_register_actions),
+                ("feature-discovered", ged.GUI1, self._on_feature_discovered),
             ]
         )
 
@@ -199,7 +202,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         return self._message_action_box
 
     def get_message_input(self) -> MessageInputTextView:
-        return self._message_action_box.msg_textview
+        return self._message_action_box.get_message_input()
 
     def show_chat(self, account: str, jid: JID) -> None:
         # Store (preserve) primary clipboard and restore it after switching
@@ -221,8 +224,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         )
         self._current_contact = contact
 
-        app.preview_manager.clear_previews()
-
         self._chat_banner.switch_contact(self._current_contact)
         self._chat_control.switch_contact(self._current_contact)
         self._message_action_box.switch_contact(self._current_contact)
@@ -285,7 +286,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         app.plugin_manager.extension_point("switch_contact", self._current_contact)
 
-        GLib.idle_add(self._message_action_box.msg_textview.grab_focus_delayed)
+        idle_add_once(app.window.activate_action, "win.input-focus")
 
     def _on_primary_clipboard_read_text_finished(
         self,
@@ -314,37 +315,31 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_room_password_required(
         self, _contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
 
     def _on_room_captcha_challenge(
         self, contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
 
     def _on_room_captcha_error(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error)
 
     def _on_room_creation_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CREATION_FAILED, error)
 
     def _on_room_join_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.JOIN_FAILED, error)
 
     def _on_room_config_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CONFIG_FAILED)
 
     def _on_muc_state_changed(
@@ -359,7 +354,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _user_contact: GroupchatParticipant,
         _event: events.MUCUserJoined,
     ) -> None:
-
         self._update_group_chat_actions(contact)
 
     def _on_user_role_changed(
@@ -369,7 +363,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _user_contact: GroupchatParticipant,
         _event: events.MUCUserRoleChanged,
     ) -> None:
-
         self._update_group_chat_actions(contact)
 
     def _on_user_affiliation_changed(
@@ -390,10 +383,19 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         self._update_group_chat_actions(self._current_contact)
 
+    def _on_feature_discovered(self, event: events.FeatureDiscovered) -> None:
+        if self._current_contact is None:
+            return
+
+        if event.account != self._current_contact.account:
+            return
+
+        if event.feature == Namespace.HTTPUPLOAD_0:
+            self._update_base_actions(self._current_contact)
+
     def _on_account_state(
         self, event: events.AccountConnected | events.AccountDisconnected
     ) -> None:
-
         if self._current_contact is None:
             return
 
@@ -454,26 +456,18 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         self._issue_notification(event.account, message)
 
-    def _issue_notification(self, account: str, data: Message) -> None:
-
-        text = data.text
+    def _issue_notification(self, account: str, message: Message) -> None:
+        text = message.text
         assert text is not None
 
         client = app.get_client(account)
-        contact = client.get_module("Contacts").get_contact(data.remote.jid)
+        contact = client.get_module("Contacts").get_contact(message.remote.jid)
 
         title = _("New message from")
 
-        is_previewable = app.preview_manager.is_previewable(text, data.oob)
-        if is_previewable:
-            scheme = urlparse(text).scheme
-            if scheme == "geo":
-                location = split_geo_uri(text)
-                text = format_geo_coords(float(location.lat), float(location.lon))
-            else:
-                file_name = filename_from_uri(text)
-                _icon, file_type = guess_simple_file_type(text)
-                text = f"{file_type} ({file_name})"
+        preview = get_preview_data(text, message.oob)
+        if preview is not None:
+            text = preview.text
 
         sound: str | None = None
         msg_type = "chat-message"
@@ -485,7 +479,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         if isinstance(contact, GroupchatContact):
             msg_type = "group-chat-message"
-            title += f" {data.resource} ({contact.name})"
+            title += f" {message.resource} ({contact.name})"
             assert contact.nickname is not None
             needs_highlight = message_needs_highlight(
                 text, contact.nickname, client.get_own_jid().bare
@@ -512,13 +506,13 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
             if text.startswith("/me "):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
-                    name = data.resource
+                    name = message.resource
                 text = f"* {name} {text[3:]}"
         else:
             text = ""
 
         if isinstance(contact, GroupchatContact):
-            resource = data.resource
+            resource = message.resource
         else:
             resource = None
 
@@ -541,7 +535,7 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         online = app.account_is_connected(contact.account)
 
         app.window.get_action("send-message").set_enabled(
-            allow_send_message(self._message_action_box.msg_textview.has_text, contact)
+            allow_send_message(self.get_message_input().has_text, contact)
         )
 
         httpupload = app.window.get_action("send-file-httpupload")
@@ -594,7 +588,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         pass
 
     def _on_action(self, action: Gio.SimpleAction, param: GLib.Variant | None) -> None:
-
         if self.get_visible_child_name() != "controls":
             return
 
@@ -755,10 +748,24 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_drag_leave(self, _drop_target: Gtk.DropTarget) -> None:
         self._drop_area.set_visible(False)
 
-    def _on_drop_accept(self, _target: Gtk.DropTarget, _drop: Gdk.Drop) -> bool:
+    def _on_drop_accept(self, _target: Gtk.DropTarget, drop: Gdk.Drop) -> bool:
+        # Don't allow dropping certain gtypes directly into chat
+        drag = drop.get_drag()
+        if drag is not None:
+            formats = drag.get_formats()
+            if (
+                formats.contain_gtype(PreviewWidget)
+                or formats.contain_gtype(ChatListRow)
+                or formats.contain_gtype(SideBarListBoxRow)
+            ):
+                return False
+
         # DND on X11 freezes due to a GTK bug:
         # https://dev.gajim.org/gajim/gajim/-/issues/12313
-        return not app.is_display(Display.X11)
+        if app.is_display(Display.X11):
+            return gi_gui_package_version("Gtk>=4.20.1")
+
+        return True
 
     def _on_file_drop(
         self,
@@ -767,7 +774,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _x: float,
         _y: float,
     ) -> bool:
-
         if value is None:
             log.debug("Drop received, but value is None")
             return False
@@ -834,7 +840,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         data: str | None = None,
         files: list[str] | None = None,
     ) -> None:
-
         assert self._current_contact is not None
         self._chat_function_page.set_mode(
             self._current_contact, function_mode, data, files
@@ -857,9 +862,33 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_function_message(
         self, _function_page: ChatFunctionPage, message: str
     ) -> None:
-
         self._chat_control.add_info_message(message)
 
+    def check_send_preconditons(self) -> bool:
+        contact = self._current_contact
+        assert contact is not None
+
+        client = app.get_client(contact.account)
+
+        encryption = contact.settings.get("encryption")
+        if not encryption:
+            return True
+
+        if encryption == "OMEMO":
+            return client.get_module("OMEMO").check_send_preconditions(contact)
+
+        if encryption not in app.plugin_manager.encryption_plugins:
+            InformationAlertDialog(
+                _("Encryption Error"), _("Missing necessary encryption plugin")
+            )
+            return False
+
+        self._chat_control.sendmessage = True
+        app.plugin_manager.extension_point(
+            "send_message" + encryption, self._chat_control
+        )
+        return self._chat_control.sendmessage
+
     def _on_send_message(self) -> None:
         message = self._message_action_box.get_text()
         if message.startswith("//"):
@@ -871,24 +900,8 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         client = app.get_client(contact.account)
 
-        encryption = contact.settings.get("encryption")
-        if encryption == "OMEMO":
-            if not client.get_module("OMEMO").check_send_preconditions(contact):
-                return
-
-        elif encryption:
-            if encryption not in app.plugin_manager.encryption_plugins:
-                InformationAlertDialog(
-                    _("Encryption Error"), _("Missing necessary encryption plugin")
-                )
-                return
-
-            self._chat_control.sendmessage = True
-            app.plugin_manager.extension_point(
-                "send_message" + encryption, self._chat_control
-            )
-            if not self._chat_control.sendmessage:
-                return
+        if not self.check_send_preconditons():
+            return
 
         message = remove_invalid_xml_chars(message)
         if message in ("", "\n"):
@@ -933,8 +946,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         if self._current_contact is not None:
             self._current_contact.disconnect_all_from_obj(self)
 
-        app.preview_manager.clear_previews()
-
         self._last_quoted_id = None
         self.set_visible_child_name("empty")
         self._chat_banner.clear()
@@ -944,7 +955,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_command_signal(
         self, _chat_commands: ChatCommands, signal_name: str, text: str
     ) -> None:
-
         is_error = signal_name != "command-result"
         self._chat_control.add_command_output(text, is_error)
 
