@@ -164,6 +164,7 @@ class MessageRowActions(Gtk.Box):
     def switch_contact(self, contact: ChatContactT) -> None:
         self._message_row = None
         self._contact = contact
+        self.set_visible(False)
 
     def _hide_with_timeout(self) -> None:
         if self._timeout_id is not None:
@@ -188,7 +189,6 @@ class MessageRowActions(Gtk.Box):
         _x: int,
         _y: int,
     ) -> None:
-
         self._has_cursor = True
 
         if self._is_menu_open:
@@ -226,7 +226,7 @@ class MessageRowActions(Gtk.Box):
 
     def _on_reaction_added(self, _widget: EmojiChooser, emoji: str) -> None:
         # Remove emoji variant selectors
-        emoji = emoji.strip("\uFE0E\uFE0F")
+        emoji = emoji.strip("\ufe0e\ufe0f")
         self._send_reaction(emoji, toggle=False)
 
     def _send_reaction(self, emoji: str, toggle: bool = True) -> None:
@@ -256,12 +256,11 @@ class MessageRowActions(Gtk.Box):
 
 class QuickReactionButton(Gtk.Button):
     def __init__(self, emoji: str) -> None:
-
         self.emoji = emoji
 
         # Add emoji presentation selector, otherwise depending on the font
         # emojis might be displayed in its text variant
-        emoji_presentation_form = f"{emoji}\uFE0F"
+        emoji_presentation_form = f"{emoji}\ufe0f"
 
         Gtk.Button.__init__(
             self,
@@ -449,17 +448,18 @@ class AvatarBox(Gtk.Box, SignalManager):
         _n_press: int,
         x: float,
         y: float,
-    ) -> int:
+    ) -> None:
+        gesture_click.set_state(Gtk.EventSequenceState.CLAIMED)
         if not isinstance(self._contact, GroupchatContact):
-            return Gdk.EVENT_STOP
+            return
 
         if gesture_click.get_current_button() == Gdk.BUTTON_PRIMARY:
-            app.window.activate_action("win.mention", GLib.Variant("s", self._name))
+            app.window.activate_action(
+                "win.input-mention", GLib.Variant("s", self._name)
+            )
         elif gesture_click.get_current_button() == Gdk.BUTTON_SECONDARY:
             self._show_participant_menu(self._name, x, y)
 
-        return Gdk.EVENT_STOP
-
     def _show_participant_menu(self, nick: str, x: float, y: float) -> None:
         assert isinstance(self._contact, GroupchatContact)
         if not self._contact.is_joined:
