@@ -208,6 +208,11 @@ class ConversationView(Gtk.ScrolledWindow):
         self._contact = None
         self._client = None
 
+    def reload(self) -> None:
+        if self._contact is None:
+            return
+        self.switch_contact(self._contact)
+
     def switch_contact(self, contact: ChatContactT) -> None:
         self._contact = contact
         self._client = app.get_client(contact.account)
@@ -821,7 +826,11 @@ class ConversationView(Gtk.ScrolledWindow):
         if previous_flags & Gtk.StateFlags.PRELIGHT:
             self._message_row_actions.hide_actions()
         else:
-            success, point = row.compute_point(self, Graphene.Point.zero())
+            try:
+                success, point = row.compute_point(self, Graphene.Point.zero())
+            except GLib.Error:
+                return
+
             if not success:
                 return
 
