@@ -51,6 +51,7 @@ from gajim.gtk.chat_stack import ChatStack
 from gajim.gtk.const import MAIN_WIN_ACTIONS
 from gajim.gtk.emoji_chooser import EmojiChooser
 from gajim.gtk.main_stack import MainStack
+from gajim.gtk.preview.preview import PreviewWidget
 from gajim.gtk.start_chat import parse_uri
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import actionmethod
@@ -130,12 +131,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         controller.connect("motion", self._on_window_motion_notify)
         self.add_controller(controller)
 
-        controller = Gtk.EventControllerKey(
-            propagation_phase=Gtk.PropagationPhase.BUBBLE
-        )
-        controller.connect("key-pressed", self._on_key_pressed)
-        self.add_controller(controller)
-
         self.register_events(
             [
                 ("message-received", ged.GUI1, self._on_message_received),
@@ -171,6 +166,9 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
                 "resume-successful", self._on_client_resume_successful
             )
 
+        manager = app.app.get_shortcut_manager()
+        manager.install_shortcuts(self, "main-win")
+
     @property
     def about_dialog(self) -> AboutDialog:
         assert self._about_dialog is not None
@@ -232,7 +230,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _mark_workspace_as_read(
         self, _action: Gio.SimpleAction, param: GLib.Variant
     ) -> None:
-
         workspace_id = param.get_string() or None
         if workspace_id is not None:
             self.mark_workspace_as_read(workspace_id)
@@ -267,29 +264,13 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         self._main_stack.remove_account_page(event.account)
         self._main_stack.remove_chats_for_account(event.account)
 
-    def _on_key_pressed(
-        self,
-        _event_controller_key: Gtk.EventControllerKey,
-        keyval: int,
-        _keycode: int,
-        state: Gdk.ModifierType,
-    ) -> bool:
-
-        if keyval == Gdk.KEY_space:
-            self.get_chat_stack().get_message_input().grab_focus()
-            return Gdk.EVENT_STOP
-
-        return Gdk.EVENT_PROPAGATE
-
     def _on_client_state_changed(
         self, client: Client, _signal_name: str, state: SimpleClientState
     ) -> None:
-
         app.app.set_account_actions_state(client.account, state.is_connected)
         app.app.update_app_actions_state()
 
     def _on_client_resume_successful(self, client: Client, _signal_name: str) -> None:
-
         app.app.update_feature_actions_state(client.account)
 
     @staticmethod
@@ -304,7 +285,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
     @staticmethod
     def _on_plain_connection(event: events.PlainConnection) -> None:
-
         def _on_response(response_id: str) -> None:
             if response_id == "connect":
                 event.connect()
@@ -479,7 +459,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_action(
         self, action: Gio.SimpleAction, _param: GLib.Variant | None
     ) -> int | None:
-
         action_name = action.get_name()
         log.info("Activate action: %s", action_name)
 
@@ -517,8 +496,10 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
                 self._chat_page.remove_chat(contact.account, contact.jid)
                 return None
 
-        if action_name in ("escape", "close-chat"):
-            # Attempt to close Gajim window if no chat has been selected
+        if action_name == "escape" and app.settings.get("escape_key_closes"):
+            self.close()
+
+        elif action_name == "close-chat":
             self.close()
 
         elif action_name == "restore-chat":
@@ -549,7 +530,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_app_font_size_action(
         self, action: Gio.SimpleAction, _param: GLib.Variant
     ) -> None:
-
         action_name = action.get_name()
         if action_name == "reset-app-font-size":
             app.settings.set_app_setting("app_font_size", None)
@@ -573,7 +553,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         app.css_config.apply_app_font_size()
 
     def _on_copy_message(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         self.get_clipboard().set(param.get_string())
 
     @actionmethod
@@ -591,7 +570,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_muc_user_block(
         self, _action: Gio.SimpleAction, params: OccupantParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             client.get_module("MucBlocking").set_block_occupants(
@@ -623,7 +601,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_muc_user_unblock(
         self, _action: Gio.SimpleAction, params: OccupantParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             client.get_module("MucBlocking").set_block_occupants(
@@ -641,7 +618,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_retract_message(
         self, _action: Gio.SimpleAction, params: RetractMessageParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             contact = client.get_module("Contacts").get_contact(params.jid)
@@ -664,7 +640,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_moderate_message(
         self, _action: Gio.SimpleAction, params: ModerateMessageParam
     ) -> None:
-
         def _on_response(reason: str) -> None:
             client = app.get_client(params.account)
             client.get_module("MUC").moderate_messages(
@@ -726,7 +701,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_delete_message_locally(
         self, _action: Gio.SimpleAction, params: DeleteMessageParam
     ) -> None:
-
         def _on_response() -> None:
             app.storage.archive.delete_message(params.pk)
             app.ged.raise_event(
@@ -801,9 +775,8 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             assert contact is not None
             client = app.get_client(contact.account)
             chat_stack = self._chat_page.get_chat_stack()
-            msg_action_box = chat_stack.get_message_action_box()
             client.get_module("Chatstate").set_mouse_activity(
-                contact, msg_action_box.msg_textview.has_text
+                contact, chat_stack.get_message_input().has_text
             )
 
     def _on_chat_selected(self, *args: Any) -> None:
@@ -820,7 +793,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         elif action == "minimize":
             self.minimize()
         else:
-            self.quit()
+            app.app.start_shutdown()
 
         return Gdk.EVENT_STOP
 
@@ -857,15 +830,15 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             return False
         return self._chat_page.is_chat_selected(account, jid)
 
-    def highlight_dnd_targets(self, drag_row: Any, highlight: bool) -> None:
-        if isinstance(drag_row, ChatListRow):
+    def highlight_dnd_targets(self, dragged_object: Any, highlight: bool) -> None:
+        if isinstance(dragged_object, ChatListRow | PreviewWidget):
             chat_list_stack = self._chat_page.get_chat_list_stack()
             workspace = self.get_active_workspace()
             if workspace is None:
                 return
 
-            if drag_row.is_pinned:
-                chat_list = chat_list_stack.get_chatlist(workspace)
+            chat_list = chat_list_stack.get_chatlist(workspace)
+            if isinstance(dragged_object, ChatListRow) and dragged_object.is_pinned:
                 for row in chat_list.get_chat_list_rows():
                     if not row.is_pinned:
                         continue
@@ -875,10 +848,16 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
                     else:
                         row.remove_css_class("dnd-target-chatlist")
 
-        self._app_side_bar.highlight_dnd_targets(highlight)
+            if isinstance(dragged_object, PreviewWidget):
+                for row in chat_list.get_chat_list_rows():
+                    if highlight:
+                        row.add_css_class("dnd-target-chatlist")
+                    else:
+                        row.remove_css_class("dnd-target-chatlist")
+
+        self._app_side_bar.highlight_dnd_targets(dragged_object, highlight)
 
     def _add_workspace(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         workspace_id = param.get_string()
         if workspace_id:
             self.add_workspace(workspace_id)
@@ -886,7 +865,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def add_workspace(
         self, workspace_id: str | None = None, switch: bool = True
     ) -> str:
-
         if workspace_id is None:
             workspace_id = app.settings.add_workspace(_("My Workspace"))
 
@@ -906,7 +884,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         open_window("WorkspaceDialog", workspace_id=workspace_id)
 
     def _remove_workspace(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         workspace_id = param.get_string() or None
         if workspace_id is None:
             workspace_id = self.get_active_workspace()
@@ -964,7 +941,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _activate_workspace(
         self, _action: Gio.SimpleAction, param: GLib.Variant
     ) -> None:
-
         workspace_id = param.get_string()
         if workspace_id:
             self.activate_workspace(workspace_id)
@@ -986,7 +962,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _get_suitable_workspace(
         self, account: str, jid: JID, private_chat: bool = False
     ) -> str:
-
         if private_chat:
             # Try to add private chat to the same workspace the MUC resides in
             chat_list_stack = self._chat_page.get_chat_list_stack()
@@ -1005,19 +980,16 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         return self._app_side_bar.get_first_workspace()
 
     def _add_group_chat(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         account, jid, select = param.unpack()
         self.add_group_chat(account, JID.from_string(jid), select)
 
     def add_group_chat(self, account: str, jid: JID, select: bool = False) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, "groupchat", select=select
         )
 
     def _add_chat(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         params = AddChatActionParams.from_variant(param)
         self.add_chat(params.account, params.jid, params.type, params.select)
 
@@ -1029,14 +1001,12 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         select: bool = False,
         message: str | None = None,
     ) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, type_, select=select, message=message
         )
 
     def add_private_chat(self, account: str, jid: JID, select: bool = False) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid, private_chat=True)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, "pm", select=select
@@ -1080,7 +1050,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def is_message_correctable(
         self, contact: types.ChatContactT, message_id: str
     ) -> bool:
-
         chat_stack = self._chat_page.get_chat_stack()
         last_message_id = chat_stack.get_last_message_id(contact)
         if last_message_id is None or last_message_id != message_id:
@@ -1109,7 +1078,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         *,
         is_sync: bool = False,
     ) -> None:
-
         unread_count = self.get_chat_unread_count(account, jid, include_silent=True)
 
         self.set_urgency_hint(False)
@@ -1165,7 +1133,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             self.mark_as_read(contact.account, contact.jid)
 
     def get_preferred_ft_method(self, contact: types.ChatContactT) -> str | None:
-
         httpupload_enabled = app.window.get_action_enabled("send-file-httpupload")
         return "httpupload" if httpupload_enabled else None
         # TODO Jingle FT
@@ -1198,7 +1165,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         nickname: str | None = None,
         password: str | None = None,
     ) -> None:
-
         jid_ = JID.from_string(jid)
         if not self.chat_exists(account, jid_):
             client = app.get_client(account)
@@ -1209,7 +1175,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def start_chat_from_jid(
         self, account: str, jid: str, message: str | None = None
     ) -> None:
-
         jid_ = JID.from_string(jid)
         if self.chat_exists(account, jid_):
             self.select_chat(account, jid_)
@@ -1295,7 +1260,20 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         self._set_startup_finished()
 
+    def _is_history_sync(self, event: events.MessageReceived) -> bool:
+        if event.mam is None:
+            return False
+
+        win = get_app_window("HistorySyncAssistant", account=event.account)
+        if win is None:
+            return False
+
+        return win.get_active_query_id() == event.mam.query_id
+
     def _on_message_received(self, event: events.MessageReceived) -> bool | None:
+        if self._is_history_sync(event):
+            return ged.STOP_PROPAGATION
+
         if self.chat_exists(event.account, event.jid):
             return
 
@@ -1348,7 +1326,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         if not self.chat_exists(event.account, event.jid):
             self.add_chat(event.account, event.jid, "chat")
 
-    def quit(self) -> None:
+    def start_shutdown(self) -> None:
         self.show_toast(Adw.Toast(title=_("Gajim is quitting…"), timeout=0))
 
         if self.is_visible():
@@ -1356,5 +1334,5 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             app.settings.set("mainwin_width", window_width)
             app.settings.set("mainwin_height", window_height)
 
-        app.settings.save()
-        app.app.start_shutdown()
+    def reload_view(self, *args: Any) -> None:
+        self.get_control().get_conversation_view().reload()
