@@ -39,7 +39,7 @@ from gajim.gtk.sidebar_switcher import SideBarMenuItem
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import iterate_listbox_children
 from gajim.gtk.util.window import open_window
-from gajim.gtk.widgets import GajimAppWindow
+from gajim.gtk.window import GajimAppWindow
 
 log = logging.getLogger("gajim.gtk.settings")
 
@@ -60,15 +60,15 @@ class SettingsDialog(GajimAppWindow):
             title=title,
             default_width=250,
             transient_for=parent,
-            add_window_padding=False,
+            header_bar=True,
         )
 
         self.account = account
         if flags == Gtk.DialogFlags.MODAL:
-            self.window.set_modal(True)
+            self.set_modal(True)
 
         elif flags == Gtk.DialogFlags.DESTROY_WITH_PARENT:
-            self.window.set_destroy_with_parent(True)
+            self.set_destroy_with_parent(True)
 
         self.listbox = SettingsBox(account, extend=extend)
         self.listbox.set_hexpand(True)
@@ -743,7 +743,7 @@ class DropDownSetting(GenericSetting):
         box.set_halign(Gtk.Align.END)
         box.set_hexpand(True)
 
-        self._dropdown = GajimDropDown(data=data, fixed_width=15)
+        self._dropdown: GajimDropDown[Any] = GajimDropDown(data=data, fixed_width=15)
         self._dropdown.select_key(self.setting_value)
         self._dropdown.connect("notify::selected", self._on_selected)
         box.append(self._dropdown)
@@ -758,10 +758,10 @@ class DropDownSetting(GenericSetting):
         app.check_finalize(self._dropdown)
         del self._dropdown
 
-    def _on_selected(self, dropdown: GajimDropDown, *args: Any) -> None:
+    def _on_selected(self, dropdown: GajimDropDown[Any], *args: Any) -> None:
         item = dropdown.get_selected_item()
         assert item is not None
-        self.set_value(item.props.key)
+        self.set_value(item.key)
 
     def update_entries(self, data: list[str] | dict[str, str]) -> None:
         self._dropdown.disconnect_by_func(self._on_selected)
